import pecan

from oslo_log import log as logging
from wsme import types
from wsmeext.pecan import wsexpose as expose

from magellan.api.controllers import resource
from magellan.api.controllers.v1 import root as v1_root

LOG = logging.getLogger(__name__)
API_STATUS = types.Enum(str, 'SUPPORTED', 'CURRENT', 'DEPRECATED')


class APIVersion(types.Base):
    """An API Version."""

    id = types.text
    "The version identifier."

    status = API_STATUS
    "The status of the API (SUPPORTED, CURRENT or DEPRECATED)."

    links = types.ArrayType(resource.Link)
    "The link to the versioned API."

    @classmethod
    def sample(cls):
        return cls(
            id='v1.0',
            status='CURRENT',
            links=[
                resource.Link(target_name='v1', rel="self",
                              href='http://example.com:8008/v1')
            ]
        )


class APIVersions(types.Base):
    """API Versions."""
    versions = types.ArrayType(APIVersion)

    @classmethod
    def sample(cls):
        v1 = APIVersion(id='v1', status='CURRENT', rel="self",
                        href='http://example.com:8008/v1')
        return cls(versions=[v1])


class RootController(object):
    v1 = v1_root.Controller()

    @expose(APIVersions)
    def index(self):
        LOG.debug("Fetching API versions.")

        host_url_v1 = '%s/%s' % (pecan.request.host_url, 'v1')
        api_v1 = APIVersion(
            id='v2.0',
            status='CURRENT',
            links=[resource.Link(href=host_url_v1, target='v1',
                                 rel="self", )]
        )

        return APIVersions(versions=[api_v1])
