import eventlet
import pickle
import base64

from oslo_config import cfg
import oslo_messaging
from oslo_log import log as logging


LOG = logging.getLogger(__name__)
TRANSPORT = None
NOTIFIER = None


class MagellanSerializer(oslo_messaging.NoOpSerializer):

    # TODO: We need to consider better serialization

    def serialize_entity(self, ctxt, entity):
        return base64.b64encode(pickle.dumps(entity))

    def deserialize_entity(self, ctxt, entity):
        return pickle.loads(base64.b64decode(entity))


def setup(url=None, optional=False):
    """Initialise the oslo_messaging layer."""
    global TRANSPORT, NOTIFIER

    if url and url.startswith("fake://"):
        # NOTE: oslo_messaging fake driver uses time.sleep
        # for task switch, so we need to monkey_patch it
        eventlet.monkey_patch(time=True)

    if url is None:
        url = cfg.CONF.messaging.transport_url
    if not TRANSPORT:
        oslo_messaging.set_transport_defaults(cfg.CONF.messaging.exchange)
        exmods = ['magellan.common.exceptions']
        try:
            TRANSPORT = oslo_messaging.get_transport(
                cfg.CONF, url, allowed_remote_exmods=exmods)
        except oslo_messaging.InvalidTransportURL as e:
            TRANSPORT = None
            if not optional or e.url:
                raise

    if not NOTIFIER and TRANSPORT:
        serializer = MagellanSerializer()
        NOTIFIER = oslo_messaging.Notifier(TRANSPORT, serializer=serializer)


def cleanup():
    """Cleanup the oslo_messaging layer."""
    global TRANSPORT, NOTIFIER
    if TRANSPORT:
        TRANSPORT.cleanup()
        TRANSPORT = NOTIFIER = None


def get_rpc_server(target, endpoint):
    """Return a configured oslo_messaging rpc server."""
    serializer = MagellanSerializer()
    return oslo_messaging.get_rpc_server(TRANSPORT, target, [endpoint],
                                         executor='eventlet',
                                         serializer=serializer,
                                         access_policy=None)


def get_rpc_client(**kwargs):
    """Return a configured oslo_messaging RPCClient."""
    target = oslo_messaging.Target(**kwargs)
    serializer = MagellanSerializer()
    return oslo_messaging.RPCClient(TRANSPORT, target, serializer=serializer)


def get_notifier(publisher_id):
    """Return a configured oslo_messaging notifier."""
    return NOTIFIER.prepare(publisher_id=publisher_id)
