import eventlet
from oslo_service import threadgroup


class ThreadManager():
    """Thread manager."""

    def __init__(self):

        super()
        self.threads = {}
        self.group = threadgroup.ThreadGroup()

        # Create dummy service task, because when there is nothing queued
        # on self.tg the process exits
        self.add_timer(900, self._service_task)

    def _service_task(self):
        """
        Dummy task which gets queued on the service.Service threadgroup.
        Without this service.Service sees nothing running i.e has nothing to
        wait() on, so the process exits..
        This could also be used to trigger periodic non-cluster-specific
        housekeeping tasks
        """
        pass

    def start(self, func, *args, **kwargs):
        """Run the given method in a sub-thread."""
        return self.group.add_thread(func, *args, **kwargs)

    def add_timer(self, interval, func, *args, **kwargs):
        """
        Define a periodic task to be run in the thread group.
        The task will be executed in a separate green thread.
        """
        self.group.add_timer(interval, func, *args, **kwargs)

    def stop_timers(self):
        self.group.stop_timers()

    def stop(self, graceful=False):
        """
        Stop any active threads belong to this threadgroup.
        """
        # Try to stop all threads gracefully
        self.group.stop(graceful)
        self.group.wait()

        # Wait for link()ed functions (i.e. lock release)
        threads = self.group.threads[:]
        links_done = dict((th, False) for th in threads)

        def mark_done(gt, th):
            links_done[th] = True

        for th in threads:
            th.link(mark_done, th)

        while not all(links_done.values()):
            eventlet.sleep()
