import sqlalchemy

from oslotest import base
from oslo_config import cfg
from oslo_db import options

from magellan.db import api as api
from magellan.db.sqlalchemy import api as db_api
from magellan.common import config

get_engine = api.get_engine


class MagellanTestCase(base.BaseTestCase):
    """Test case base class for all unit tests."""

    def override_config(self, name, override, group=None):
        """Cleanly override CONF variables."""
        cfg.CONF.set_override(name, override, group)
        self.addCleanup(cfg.CONF.clear_override, name, group)


class SqlAlchemyTest(MagellanTestCase):
    @staticmethod
    def setup_dummy_db():
        config.set_config_defaults()
        options.cfg.set_defaults(options.database_opts,
                                 sqlite_synchronous=False)
        options.set_defaults(cfg.CONF, connection="sqlite://")
        engine = get_engine()
        db_api.db_sync(engine)
        engine.connect()
        # With sqlite we must manually enable foreign_keys to get integrity
        # errors like we would in a production environment using MariaDB.
        engine.execute("PRAGMA foreign_keys=ON;")

    @staticmethod
    def reset_dummy_db():
        engine = get_engine()
        meta = sqlalchemy.MetaData()
        meta.reflect(bind=engine)

        for table in reversed(meta.sorted_tables):
            if table.name == 'migrate_version':
                continue
            engine.execute(table.delete())

    def setUp(self):
        super(SqlAlchemyTest, self).setUp()

        SqlAlchemyTest.setup_dummy_db()
        self.addCleanup(SqlAlchemyTest.reset_dummy_db)
