from oslo_config import cfg
import mock
import pecan
import pecan.testing
from webtest import app as webtest_app

from magellan.tests import base
from magellan.common import messaging

# Disable authentication for API tests.
cfg.CONF.set_default('auth_enable', False, group='pecan')


class APITest(base.SqlAlchemyTest):

    @mock.patch.object(messaging, "get_rpc_client", mock.Mock())
    def setUp(self):
        super(APITest, self).setUp()

        pecan_opts = cfg.CONF.pecan

        self.app = pecan.testing.load_test_app({
            'app': {
                'root': pecan_opts.root,
                'modules': pecan_opts.modules,
                'debug': pecan_opts.debug,
                'auth_enable': False,
            }
        })

        self.addCleanup(pecan.set_config, {}, overwrite=True)
        self.addCleanup(cfg.CONF.set_default,
                        'auth_enable',
                        False,
                        group='pecan')

    def assertNotFound(self, url):
        try:
            self.app.get(url, headers={'Accept': 'application/json'})
        except webtest_app.AppError as error:
            self.assertIn('Bad response: 404 Not Found', str(error))

            return

        self.fail('Expected 404 Not found but got OK')
