import mock
from oslo_concurrency import processutils
from oslo_config import cfg
import pecan.testing

from magellan.api import service
from magellan.tests import base


class TestWSGIService(base.MagellanTestCase):
    @mock.patch('magellan.api.app.setup_app')
    @mock.patch.object(service.wsgi, 'Server')
    def test_workers_set_default(self, wsgi_server, mock_app):
        service_name = "magellan_api"
        mock_app.return_value = pecan.testing.load_test_app({
            'app': {
                'root': cfg.CONF.pecan.root,
                'modules': cfg.CONF.pecan.modules,
                'debug': cfg.CONF.pecan.debug,
                'auth_enable': cfg.CONF.pecan.auth_enable
            }
        })
        test_service = service.WSGIService(service_name)

        wsgi_server.assert_called_once_with(
            cfg.CONF,
            service_name,
            test_service.app,
            host='0.0.0.0',
            port=8008,
            use_ssl=False
        )

    @mock.patch('magellan.api.app.setup_app')
    @mock.patch.object(service.wsgi, 'Server')
    def test_workers_set_correct_setting(self, wsgi_server, mock_app):
        self.override_config('api_workers', 8, group='api')

        mock_app.return_value = pecan.testing.load_test_app({
            'app': {
                'root': cfg.CONF.pecan.root,
                'modules': cfg.CONF.pecan.modules,
                'debug': cfg.CONF.pecan.debug,
                'auth_enable': cfg.CONF.pecan.auth_enable
            }
        })
        test_service = service.WSGIService("magellan_api")

        self.assertEqual(8, test_service.workers)

    @mock.patch('magellan.api.app.setup_app')
    @mock.patch.object(service.wsgi, 'Server')
    def test_workers_set_zero_setting(self, wsgi_server, mock_app):
        self.override_config('api_workers', 0, group='api')

        mock_app.return_value = pecan.testing.load_test_app({
            'app': {
                'root': cfg.CONF.pecan.root,
                'modules': cfg.CONF.pecan.modules,
                'debug': cfg.CONF.pecan.debug,
                'auth_enable': cfg.CONF.pecan.auth_enable
            }
        })
        test_service = service.WSGIService("magellan_api")

        self.assertEqual(processutils.get_worker_count(), test_service.workers)

    @mock.patch('magellan.api.app.setup_app')
    @mock.patch.object(service.wsgi, 'Server')
    def test_wsgi_service_with_ssl_enabled(self, wsgi_server, mock_app):
        self.override_config('enable_ssl_api', True, group='api')

        mock_app.return_value = pecan.testing.load_test_app({
            'app': {
                'root': cfg.CONF.pecan.root,
                'modules': cfg.CONF.pecan.modules,
                'debug': cfg.CONF.pecan.debug,
                'auth_enable': cfg.CONF.pecan.auth_enable
            }
        })
        service_name = 'magellan_api'
        srv = service.WSGIService(service_name)

        wsgi_server.assert_called_once_with(
            cfg.CONF,
            service_name,
            srv.app,
            host='0.0.0.0',
            port=8008,
            use_ssl=True
        )
