
import unittest

from wsme import types
from magellan.api.controllers.v1 import usertypes


class A(types.Base):
    key = int


class B(types.Base):
    key = types.text


class TestUsertypeUnion(unittest.TestCase):

    def test_union_type_matching(self):

        union = usertypes.Union('Test Name', A, B)
        instance = union.frombasetype({'key': 123})
        assert isinstance(instance, A), 'Type is {}'.format(type(instance))
        instance = union.frombasetype({'key': 'abc'})
        assert isinstance(instance, B), 'Type is {}'.format(type(instance))

    def test_union_to_base_type(self):

        union = usertypes.Union('Test Name', A, B)
        data = union.tobasetype(A(key=123))
        assert data['key'] == 123
        data = union.tobasetype(B(key='abc'))
        assert data['key'] == 'abc'

    def test_union_error_on_no_match(self):
        with self.assertRaisesRegex(
                ValueError,
                "{'key': \[\]} does not match any variant of Test Name"):
            union = usertypes.Union('Test Name', A, B)
            union.frombasetype({'key': []})
