import eventlet
from oslo_config import cfg
from oslo_log import log as logging
from oslo_service import service

from magellan.engine import server as engine_server
from magellan.tests import base

LOG = logging.getLogger(__name__)


def launch_service(s):
    launcher = service.ServiceLauncher(cfg.CONF)

    launcher.launch_service(s)

    launcher.wait()


class EngineTestCase(base.SqlAlchemyTest):
    def setUp(self):
        super(EngineTestCase, self).setUp()

        self.threads = []

        # Start engine.
        LOG.info("Starting engine threads...")
        eng_svc = engine_server.get_oslo_service()
        self.engine = eng_svc.engine
        self.threads.append(eventlet.spawn(launch_service, eng_svc))
        self.addCleanup(eng_svc.stop, True)

        self.addOnException(self.print_executions)
        self.addCleanup(self.kill_threads)

        eng_svc.wait_started()

    def kill_threads(self):
        LOG.info("Finishing engine and executor threads...")

        for thread in self.threads:
            thread.kill()

    @staticmethod
    def print_executions(exc_info=None):
        if exc_info:
            print("\nEngine test case exception occurred: %s" % exc_info[1])
            print("Exception type: %s" % exc_info[0])

    def test_engine_is_started(self):
        self.assertIsNotNone(self.engine)
