import mock
import pytest

from magellan.rpc import server as rpc_server
from magellan.engine import server as engine_server
from magellan.common import config
from magellan.common import consts
from oslo_config import cfg

CONF = cfg.CONF


@pytest.fixture
def server_obj():
    config.set_config_defaults()
    return engine_server.get_oslo_service()


def test_init(server_obj):
    assert server_obj.rpc_server.host == CONF.engine.host
    assert server_obj.rpc_server.rpc_api_version == consts.RPC_API_VERSION
    assert server_obj.rpc_server.topic == consts.TOPIC_KB_ENGINE


@mock.patch.object(rpc_server, 'rpc_messaging')
def test_start(mock_rpc, server_obj):
    server_obj.start()
    assert server_obj.rpc_server.target.server == CONF.engine.host
    assert server_obj.rpc_server.target.version == consts.RPC_API_VERSION
    assert server_obj.rpc_server.target.topic == consts.TOPIC_KB_ENGINE
    mock_rpc.get_rpc_server.assert_called_once_with(
        server_obj.rpc_server.target, server_obj.rpc_server)
    mock_rpc.get_rpc_server().start.assert_called_once_with()


@mock.patch.object(rpc_server, 'rpc_messaging')
def test_stop(mock_rpc, server_obj):
    server_obj.start()
    server_obj.stop(graceful=True)
    mock_rpc.get_rpc_server().stop.assert_called_once_with()
