import wsme
import wsmeext.pecan

from http.client import responses


def monkeypatch_wsme():
    """
    WSME does not provide any method for customizing the data returned when
    an error occurs. This will monkey patch WSME to return error messages on
    our preferred format.
    """

    def monkeypatch_method(cls):
        def decorator(func):
            setattr(cls, func.__name__, func)
            return func
        return decorator

    @monkeypatch_method(wsme.api)
    def format_exception(excinfo, debug=False):
        error = excinfo[1]
        code = getattr(error, 'code', 500)
        return {
            'name': responses.get(code, 'Unknown Error'),
            'description':
                getattr(error, 'message', '')
                or getattr(error, 'faultstring', '')
                or getattr(error, 'value', ''),
            'errors': []
        }

    @monkeypatch_method(wsmeext.pecan.JSonRenderer)
    def render(_, template_path, namespace):
        if 'result' in namespace:
            return wsme.rest.json.encode_result(
                namespace['result'],
                namespace['datatype']
            )
        else:
            return wsme.rest.json.encode_error(None, namespace)
